package com.atlassian.elasticsearch.buckler;

import com.atlassian.elasticsearch.buckler.config.BucklerConfig;
import com.atlassian.elasticsearch.buckler.config.TlsConfig;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelPipeline;
import io.netty.handler.ssl.SslHandler;
import org.elasticsearch.Version;
import org.elasticsearch.cluster.node.DiscoveryNode;
import org.elasticsearch.common.inject.Inject;
import org.elasticsearch.common.io.stream.NamedWriteableRegistry;
import org.elasticsearch.common.network.NetworkService;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.common.util.BigArrays;
import org.elasticsearch.common.util.PageCacheRecycler;
import org.elasticsearch.indices.breaker.CircuitBreakerService;
import org.elasticsearch.threadpool.ThreadPool;
import org.elasticsearch.transport.netty4.Netty4Transport;

/**
 * TLS for TCP transport.
 */
public class SecureTransport extends Netty4Transport {

    private final BucklerConfig config;
    public static final Version version = Version.V_7_5_2;

    @Inject
    public SecureTransport(Settings settings, ThreadPool threadPool, NetworkService networkService, PageCacheRecycler pageCacheRecycler,
                           BucklerConfig config, NamedWriteableRegistry namedWriteableRegistry, CircuitBreakerService circuitBreakerService) {
        super(settings, version, threadPool, networkService, pageCacheRecycler, namedWriteableRegistry, circuitBreakerService);

        this.config = config;
    }

    @Override
    protected ChannelHandler getServerChannelInitializer(String name) {
        return config.getTlsConfig().isEnabledForTcp()
                ? new TlsServerChannelPipelineFactory(name, config)
                : super.getServerChannelInitializer(name);
    }

    @Override
    protected ChannelHandler getClientChannelInitializer(DiscoveryNode node) {
        return config.getTlsConfig().isEnabledForTcp()
                ? new TlsClientChannelPipelineFactory(config)
                : super.getClientChannelInitializer(node);
    }

    private class TlsClientChannelPipelineFactory extends ClientChannelInitializer {

        private final BucklerConfig config;

        public TlsClientChannelPipelineFactory(BucklerConfig config) {
            this.config = config;
        }

        @Override
        protected void initChannel(Channel ch) throws Exception {
            super.initChannel(ch);
            ChannelPipeline pipeline = ch.pipeline();

            TlsConfig tlsConfig = config.getTlsConfig();
            if (tlsConfig.isEnabledForTcp()) {
                SslHandler sslHandler = tlsConfig.createHandler(true);
                pipeline.addFirst("tls", sslHandler);
            }
        }
    }

    private class TlsServerChannelPipelineFactory extends ServerChannelInitializer {

        private final BucklerConfig config;

        public TlsServerChannelPipelineFactory(String name, BucklerConfig config) {
            super(name);
            this.config = config;
        }

        @Override
        protected void initChannel(Channel ch) throws Exception {
            super.initChannel(ch);
            ChannelPipeline pipeline = ch.pipeline();

            TlsConfig tlsConfig = config.getTlsConfig();
            if (tlsConfig.isEnabledForTcp()) {
                SslHandler sslHandler = tlsConfig.createHandler(false);
                pipeline.addFirst("tls", sslHandler);
            }
        }
    }
}
