package com.atlassian.elasticsearch.buckler.config;

import io.netty.handler.ssl.SslHandler;
import org.elasticsearch.common.settings.Settings;

import javax.net.ssl.KeyManager;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLEngine;
import java.io.FileInputStream;
import java.security.KeyStore;

public class TlsConfig {

    private final boolean enabledForHttp;
    private final boolean enabledForTcp;

    /**
     * Whether this node is configured to be a server. Note that even when it is a server, it can also do client connections.
     */
    private final boolean server;
    private final String keyStorePath;
    private final char[] keyStorePassword;

    TlsConfig(Settings settings, boolean server) {
        this.enabledForHttp = BucklerConfig.loadEnabled(settings, "tls.http.enabled", "TLS for HTTP transport");
        this.enabledForTcp = BucklerConfig.loadEnabled(settings, "tls.tcp.enabled", "TLS for TCP transport");
        this.server = server;
        if (server && (enabledForHttp || enabledForTcp)) {
            this.keyStorePath = BucklerConfig.getRequiredSetting(settings, "tls.keystore.path",
                    "Keystore path must be configured for TLS");
            this.keyStorePassword = BucklerConfig.getRequiredSetting(settings, "tls.keystore.password",
                    "Keystore password must be configured for TLS").toCharArray();
        } else {
            this.keyStorePath = null;
            this.keyStorePassword = new char[]{};
        }
    }

    public boolean isEnabledForHttp() {
        return enabledForHttp;
    }

    public boolean isEnabledForTcp() {
        return enabledForTcp;
    }

    public SslHandler createHandler(boolean clientMode) throws Exception {
        SSLContext sslContext = createContext();
        SSLEngine sslEngine = sslContext.createSSLEngine();
        sslEngine.setUseClientMode(clientMode);

        return new SslHandler(sslEngine);
    }

    private SSLContext createContext() throws Exception {
        KeyManager[] keyManagers;
        if (server) {
            KeyManagerFactory kmf = KeyManagerFactory.getInstance(KeyManagerFactory.getDefaultAlgorithm());
            KeyStore keys = KeyStore.getInstance(KeyStore.getDefaultType());
            try (FileInputStream inputStream = new FileInputStream(keyStorePath)) {
                keys.load(inputStream, keyStorePassword);
            }
            kmf.init(keys, keyStorePassword);
            keyManagers = kmf.getKeyManagers();
        } else {
            keyManagers = null;
        }

        SSLContext sslContext = SSLContext.getInstance("TLSv1.2");
        sslContext.init(keyManagers, null, null);
        return sslContext;
    }

}
